/**
#The Toce fluvial case
 
## Before you start
Before you start, be sure to create the following folders in the same place:
"Movies" "Pic" "Raster" "Gauges" 


To compile, type in the terminal: 
qcc Case-toce-grand1.c -o Case-toce-grand1.x -lm ~/basilisk/src/kdt/kdt.o
*/

/**
## Include and def
*/
#include "b-flood/saint-venant-topo.h"
#include "b-flood/manning.h"
#include "b-flood/inflow.h"
#include "terrain.h"
#include "input.h"

#define MAXLEVEL 10 // 1024 Volume
#define MINLEVEL 6
#define EMAXE 5e-4

int nf,nc,ncell;
timer t0;

/**
## Parameters  
*/

int main() {
  n = 0.0162;
  L0 = 43.4;
  X0 = 0.1;
  Y0 = 0;
  G = 9.81;
  dry = 1e-6;
  N = 1 << MAXLEVEL; // Which is equivalent to N = 2^(maxlevel)
  run();
}

event stop(t = 180)
  fprintf(stderr,"fini\n");
  //The case lasts 180 seconds
  

/**
## Initial condition
   
*/
event init (i = 0){
	//We record the topography in a special way (k-tree) so that we can use adaptive refinement on it.
  terrain (zb, "./Topo/topo-house", NULL);
  
  //We have to set a time step because there is no water at the beginning.
  DT = 1;
 
  t0 = timer_start();
  double dx = L0/N;

  //We set the interval through which the water flow will come in
  foreach(){
    if( y <= 8.4 && y >= 5 && x <= 8*dx)
      river[] = 1;
  }
  boundary({zb,river});
}


/**
## Inflow boundary condition

On the left, we impose the flow rate using the "discharge" function, 
 */
u.t[left] = dirichlet(0);
u.n[left] = neumann(0);

double l1, eta1, vola = 0;
event bound_left( i++ ){
  l1 =  hydrograph ( "./hydro.asc", 1);
  if( l1 < 0 )
    l1 = 0;
  eta1 = eta_b(l1,left,1);
  
  h[left] = dirichlet( max( river[] == 1 ? eta1 - zb[] : 0 , 0) );
  eta[left] = dirichlet( max( river[] == 1 ? eta1 : zb[]  , zb[]) );

  double vol = 0;
  foreach(reduction(+:vol)){
    if( h[] > dry )
      vol += h[] * sq(Delta);
  }
  double Q = 0;
  if( dt > 0 )
    Q =  (vol- vola)/dt;
  vola = vol;
 
  static FILE * fpp = fopen("eta.dat","w");
  if( i == 0)
     fprintf(fpp,"#t Q Qread EtaImp\n");
  fprintf(fpp,"%lf %lf %lf %lf \n",t,Q,l1,eta1);
  fflush(fpp);
  
}



/**
## Outflow boundary condition
We set a free exit condition on the right edge of the domain.
*/

u.t[right] = neumann(0);
u.n[right] = neumann(0);
h[right] = ( h[] > dry ? (h[] > sq(u.x[]+u.y[])/G ? dirichlet( sq(u.x[]+u.y[])/G) : neumann(0) ) : dirichlet(0) ) ;
eta[right] = ( h[] > dry ? (h[] > sq(u.x[]+u.y[])/G ? dirichlet(sq(u.x[]+u.y[])/G + zb[]) : neumann(0) ) : dirichlet(zb[]) )  ;

//We take care to reduce the artificial time step just before the water inlet starts.
event timestepreduce( t = 17 ){
  DT = 0.01;
}


/**
## Adaptative refinement

*/
int adapt_H() {
  scalar nh[];
  foreach()
    nh[] = h[] > dry ? h[] : 0;
  boundary ({nh});
  astats s = adapt_wavelet ({nh}, (double[]){EMAXE} ,MAXLEVEL,MINLEVEL);
  nc += s.nc;
  nf += s.nf;
  return s.nf;
}
event do_adapt_H (t = 17; i++){
    adapt_H();
}



//////// OUTPUT
/**
## Logfile
*/

event logfile (i += 10) {
  timing tr = timer_timing (t0, i, 0, NULL);
  stats s = statsf (h);
  scalar v[];
  foreach()
    v[] = norm(u);
  
  stats no = statsf (v);
  if( i == 0 )
    fprintf (stderr, "#1t 2treal 3i 4h.min 5h.max 6h.sum 7u.min 8u.max 9dt\n");  
  fprintf (stderr, "%g %g %d %g %g %g %g %g %g\n", t,tr.real, i, s.min, s.max, s.sum, 
	   no.min, no.max, dt);
}

Gauge gauges[] = {  
  // gauge   x         y       zb
  {"./Gauges/S2",0.280560,7.585584, "#S2"},
  {"./Gauges/S4",4.255610,7.745400, "#S4"},
  {"./Gauges/S6S",6.753537,9.831888, "#S6S"},
  {"./Gauges/S6D",7.552486,7.696863, "#S6D"},
  {"./Gauges/S8C",10.357560,10.226488, "#S8C"},
  {"./Gauges/S8D",10.743772,7.788068, "#S8D"},
  {"./Gauges/S11S",14.185498,12.916282, "#S11S"},
  {"./Gauges/P1",2.113886,7.866009, "#P1"},
  {"./Gauges/P2",3.931183,8.893101, "#P2"},
  {"./Gauges/P3",4.527122,6.587255, "#P3"},
  {"./Gauges/P4",7.086251,8.768313, "#P4"},
  {"./Gauges/P5",10.163515,10.218714, "#P5"},
  {"./Gauges/P6",10.549718,7.791065, "#P6"},
  {"./Gauges/P7",13.057081,11.047071, "#P7"},
  {"./Gauges/P8",14.950365,11.804039, "#P8"},
  {"./Gauges/P9",15.684258,13.802726, "#P9"},
  {"./Gauges/P10",18.190768,10.429219, "#P10"},
  {"./Gauges/P11",17.335485,11.745467, "#P11"},
  {"./Gauges/P12",18.124067,14.080679, "#P12"},
  {"./Gauges/P13",19.814693,11.984270, "#P13"},
  {"./Gauges/P14",20.536687,11.070315, "#P14"},
  {"./Gauges/P17",22.144704,16.609633, "#P17"},
  {"./Gauges/P18",23.848138,14.744612, "#P18"},
  {"./Gauges/P19",25.469928,16.186758, "#P19"},
  {"./Gauges/P20",28.519504,17.209109, "#P20"},
  {"./Gauges/P21",30.442363,18.357176, "#P21"},
  {"./Gauges/P22",33.546306,18.303467, "#P22"},
  {"./Gauges/P23",36.466741,18.750427, "#P23"},
  {"./Gauges/P24",39.024308,22.827062, "#P24"},
  {"./Gauges/P25",40.443587,21.238121, "#P25"},
  {"./Gauges/P26",40.965033,26.182320, "#P26"},
  {"./Gauges/P27",39.052331,20.594098, "#P27"},
  {"./Gauges/P28",40.175807,23.502085, "#P28"},
  {NULL}
};
event gauges1 (t += 1) output_gauges (gauges, {h});

/**
## Movies

We record movies of the water depth, the velocity and the level of
refinement.*/
event movies ( t= 17; t += 0.1 ) {
  scalar m[], v[], fr[];
  
  foreach(){
    v[] = norm(u);
    m[] = (h[] > dry ) - 0.5;
    fr[] = h[] > dry ? v[]/sqrt(G*h[]) : 0;
  }    
  boundary ({m,v,fr});
  
  static FILE * fph = popen ("ppm2mpeg > ./Movies/height.mpg", "w");
  output_ppm (h, fph, mask = m, min = 0, max = 0.4, n = N, linear = true);
  
  static FILE * fphtown1 = popen ("ppm2mpeg > ./Movies/height-town1.mpg", "w");
  output_ppm (h, fphtown1, mask = m, min = 0, max = 0.4, n = N, linear = true,
  	  box = {{0,5},{12.5,12.5}});
  
   static FILE * fphtown2 = popen ("ppm2mpeg > ./Movies/height-town2.mpg", "w");
  output_ppm (h, fphtown2, mask = m, min = 0, max = 0.4, n = N, linear = true,
  	  box = {{15,9},{25,19}});
  
  
  static FILE * fpv = popen ("ppm2mpeg > ./Movies/vel.mpg", "w");
  output_ppm (v, fpv, mask = m, min = 0, max = 3, n = N, linear = true);
  
  static FILE * fpfr = popen ("ppm2mpeg > ./Movies/froude.mpg", "w");
  output_ppm (fr, fpfr, mask = m, min = 0, max = 2, n = N, linear = true);
  
  scalar l = m;
  foreach()
    l[] = level;
  
  static FILE * fp1 = popen ("ppm2mpeg > ./Movies/level.mpg", "w");
  output_ppm (l, fp1, min = MINLEVEL, max = MAXLEVEL, n = N);
  
  static FILE * fp2 = popen ("ppm2mpeg > ./Movies/level-town1.mpg", "w");
  output_ppm (l, fp2, min = MINLEVEL, max = MAXLEVEL, n = N,
  	  box = {{0,5},{12.5,12.5}});
  
  static FILE * fp3 = popen ("ppm2mpeg > ./Movies/level-town2.mpg", "w");
  output_ppm (l, fp3, min = MINLEVEL, max = MAXLEVEL, n = N,
  	  box = {{15,9},{25,19}});
}

/**
## Fractal dim 

*/
int ifractal = 0;
event fracdim( t += 1){
  int ncell = 0;
  foreach(reduction(+:ncell))
    ncell++;
  static FILE * ffrac = fopen("fdim.dat","w");
  if( ifractal == 0){
    fprintf(ffrac,"# t \t i \t Ncell \t Nrefined \t Ncoarsen");
    ifractal++;
  }
  fprintf(ffrac,"%.1lf \t %i \t %i \t %i \t %i\n",t,i,ncell,nf,nc);
  nf = nc = 0;
  }

event snap(t = 65){
  static FILE * rph = fopen ("./Raster/raster_h-t65.asc", "w");
  output_grd (h, rph);

  scalar m[], v[];
    
  foreach(){
    v[] = norm(u);
    m[] = (h[] > dry ) - 0.5;
  }    
  boundary ({m,v});
  
  static FILE * rpu = fopen ("./Raster/raster_v-t65.asc", "w");
  output_grd (v, rpu);
}

event ending(t = end){
  static FILE * rph = fopen ("./Raster/raster_h.asc", "w");
  output_grd (h, rph);

  scalar m[], v[];
    
  foreach(){
    v[] = norm(u);
    m[] = (h[] > dry ) - 0.5;
  }    
  boundary ({m,v});
  
  static FILE * rpu = fopen ("./Raster/raster_v.asc", "w");
  output_grd (v, rpu);
}

event figures (t = 17; t <= 67; t += 2)
{
  scalar m[], etam[];
  foreach() {
    etam[] = eta[]*(h[] > dry);
    m[] = etam[] - zb[];
  }
  boundary ({m, etam});
  char name[80];
  sprintf (name, "./Pic/h-%g-blanc.png", t);
  output_ppm (h, min = 0, max = 0.4, mask = m, file = name, n = 1024,
	      linear = true,
	      opt = "-fill white -opaque black");
  sprintf (name, "./Pic/h-%g-noir.png", t);
  output_ppm (h, min = 0, max = 0.4, mask = m, file = name, n = 1024,
	      linear = true);
  
  
  sprintf (name, "./Pic/level-%g.png", t);
  scalar l[];
  foreach()
    l[] = level;
  output_ppm (l, min = MINLEVEL, max = MAXLEVEL, file = name, n = 1024,
	      linear = false,);
}


/**
## Results
Here is the gnuplot code to produce all the graphs of the article

~~~gnuplot GaugesP1P2P3
reset
set term png
set output 'P1P2P3.png'
load '/Users/geoffroy/mybin/gpl'
set title ''
set xlabel '' 
set ylabel 'Level (m)' 
set xtics; set ytics
set multiplot
set size .95,.35
set key bot
set origin 0.02, 0.6
plot './Gauges/P1' w lp t'P1 - numerical', \
     './Verif1/P1' w lp t'P1 - measured'
set title ''
set origin 0.02, 0.3
plot './Gauges/P2' w lp t'P2 - numerical', \
     './Verif1/P2' w lp t'P2 - measured'
set origin 0.02, .01
set xlabel 'T(s)' 
plot './Gauges/P3' w lp t'P3 - numerical', \
     './Verif1/P3' w lp t'P3 - measured'
~~~ 
 

~~~gnuplot GaugesP4P5P6
reset
set term png
set output 'P4P5P6.png'
load '/Users/geoffroy/mybin/gpl'
set xlabel '' 
set ylabel 'Level (m)' 
set xtics; set ytics
set multiplot
set size .95,.35
set key bot
set origin 0.02, 0.6
plot './Gauges/P4' w lp t'P4 - numerical', \
     './Verif1/P4' w lp t'P4 - measured'
set title ''
set origin 0.02, 0.3
plot './Gauges/P5' w lp t'P5 - numerical', \
     './Verif1/P5' w lp t'P5 - measured'
set origin 0.02, .01
set xlabel 'T(s)' 
plot './Gauges/P6' w lp t'P6 - numerical', \
     './Verif1/P6' w lp t'P6 - measured'

~~~ 

~~~gnuplot GaugesP7P8P9
reset
load '/Users/geoffroy/mybin/gpl'
set title ''
set xlabel '' 
set ylabel 'Level (m)' 
set xtics; set ytics
set multiplot
set size .95,.35
set key bot
set origin 0.02, 0.6
plot './Gauges/P7' w lp t'P7 - numerical', \
     './Verif1/P7' w lp t'P7 - measured'
set title ''
set origin 0.02, 0.3
plot './Gauges/P8' w lp t'P8 - numerical', \
     './Verif1/P8' w lp t'P8 - measured'
set origin 0.02, .01
set xlabel 'T(s)' 
plot './Gauges/P9' w lp t'P9 - numerical', \
     './Verif1/P9' w lp t'P9 - measured'

~~~ 
 

~~~gnuplot GaugesP10P11P12
reset
load '/Users/geoffroy/mybin/gpl'
set title ''
set xlabel '' 
set ylabel 'Level (m)' 
set xtics; set ytics
set multiplot
set size .95,.35
set key bot
set origin 0.02, 0.6
plot './Gauges/P10' w lp t'P10 - numerical', \
     './Verif1/P10' w lp t'P10 - measured'
set title ''
set origin 0.02, 0.3
plot './Gauges/P11' w lp t'P11 - numerical', \
     './Verif1/P11' w lp t'P11 - measured'
set origin 0.02, .01
set xlabel 'T(s)' 
plot './Gauges/P12' w lp t'P12 - numerical', \
     './Verif1/P12' w lp t'P12 - measured'

~~~ 


~~~gnuplot GaugesP13P14P15
reset
load '/Users/geoffroy/mybin/gpl'
set title ''
set xlabel '' 
set ylabel 'Level (m)' 
set xtics; set ytics
set multiplot
set size .95,.35
set key bot
set origin 0.02, 0.6
plot './Gauges/P13' w lp t'P13 - numerical', \
     './Verif1/P13' w lp t'P13 - measured'
set title ''
set origin 0.02, 0.3
plot './Gauges/P14' w lp t'P14 - numerical', \
     './Verif1/P14' w lp t'P14 - measured'
set origin 0.02, .01
set xlabel 'T(s)' 
plot './Gauges/P15' w lp t'P15 - numerical', \
     './Verif1/P15' w lp t'P15 - measured'

~~~ 


~~~gnuplot GaugesP16P17P18
reset
load '/Users/geoffroy/mybin/gpl'
set title ''
set xlabel '' 
set ylabel 'Level (m)' 
set xtics; set ytics
set multiplot
set size .95,.35
set key bot
set origin 0.02, 0.6
plot './Gauges/P16' w lp t'P16 - numerical', \
     './Verif1/P16' w lp t'P16 - measured'
set title ''
set origin 0.02, 0.3
plot './Gauges/P17' w lp t'P17 - numerical', \
     './Verif1/P17' w lp t'P17 - measured'
set origin 0.02, .01
set xlabel 'T(s)' 
plot './Gauges/P18' w lp t'P18 - numerical', \
     './Verif1/P18' w lp t'P18 - measured'

~~~ 

~~~gnuplot GaugesP19P20P21
reset
load '/Users/geoffroy/mybin/gpl'
set title ''
set xlabel '' 
set ylabel 'Level (m)' 
set xtics; set ytics
set multiplot
set size .95,.35
set key bot
set origin 0.02, 0.6
plot './Gauges/P19' w lp t'P19 - numerical', \
     './Verif1/P19' w lp t'P19 - measured'
set title ''
set origin 0.02, 0.3
plot './Gauges/P20' w lp t'P20 - numerical', \
     './Verif1/P20' w lp t'P20 - measured'
set origin 0.02, .01
set xlabel 'T(s)' 
plot './Gauges/P21' w lp t'P21 - numerical', \
     './Verif1/P21' w lp t'P21 - measured'

~~~ 

~~~gnuplot GaugesP22P23P24
reset
load '/Users/geoffroy/mybin/gpl'
set title ''
set xlabel '' 
set ylabel 'Level (m)' 
set xtics; set ytics
set multiplot
set size .95,.35
set key bot
set origin 0.02, 0.6
plot './Gauges/P22' w lp t'P22 - numerical', \
     './Verif1/P22' w lp t'P22 - measured'
set title ''
set origin 0.02, 0.3
plot './Gauges/P23' w lp t'P23 - numerical', \
     './Verif1/P23' w lp t'P23 - measured'
set origin 0.02, .01
set xlabel 'T(s)' 
plot './Gauges/P24' w lp t'P24 - numerical', \
     './Verif1/P24' w lp t'P24 - measured'

~~~ 

~~~gnuplot GaugesP25P26P27
reset
load '/Users/geoffroy/mybin/gpl'
set title ''
set xlabel '' 
set ylabel 'Level (m)' 
set xtics; set ytics
set multiplot
set size .95,.35
set key bot
set origin 0.02, 0.6
plot './Gauges/P25' w lp t'P25 - numerical', \
     './Verif1/P25' w lp t'P25 - measured'
set title ''
set origin 0.02, 0.3
plot './Gauges/P26' w lp t'P26 - numerical', \
     './Verif1/P26' w lp t'P26 - measured'
set origin 0.02, .01
set xlabel 'T(s)' 
plot './Gauges/P27' w lp t'P27 - numerical', \
     './Verif1/P27' w lp t'P27 - measured'

~~~ 


~~~gnuplot GaugesP28S2S4
reset
load '/Users/geoffroy/mybin/gpl'
set title ''
set xlabel '' 
set ylabel 'Level (m)' 
set xtics; set ytics
set multiplot
set size .95,.35
set key bot
set origin 0.02, 0.6
plot './Gauges/P28' w lp t'P28 - numerical', \
     './Verif1/P28' w lp t'P28 - measured'
set title ''
set origin 0.02, 0.3
plot './Gauges/S2' w lp t'S2 - numerical', \
     './Verif1/S2' w lp t'S2 - measured'
set origin 0.02, .01
set xlabel 'T(s)' 
plot './Gauges/S4' w lp t'S4 - numerical', \
     './Verif1/S4' w lp t'S4 - measured'

~~~ 

~~~gnuplot GaugesS6DS6SS8C
reset
load '/Users/geoffroy/mybin/gpl'
set title ''
set xlabel '' 
set ylabel 'Level (m)' 
set xtics; set ytics
set multiplot
set size .95,.35
set key bot
set origin 0.02, 0.6
plot './Gauges/S6D' w lp t'S6D - numerical', \
     './Verif1/S6D' w lp t'S6D - measured'
set title ''
set origin 0.02, 0.3
plot './Gauges/S6S' w lp t'S6S - numerical', \
     './Verif1/S6S' w lp t'S6S - measured'
set origin 0.02, .01
set xlabel 'T(s)' 
plot './Gauges/S8C' w lp t'S8C - numerical', \
     './Verif1/S8C' w lp t'S8C - measured'

~~~ 

~~~gnuplot GaugesS8D
reset
load '/Users/geoffroy/mybin/gpl'
set title ''
set xlabel 'T(s)' 
set ylabel 'Level (m)' 
set xtics; set ytics
set multiplot
set size .95,.35
set key bot
set origin 0.02, 0.6
plot './Gauges/S8D' w lp t'S8D - numerical', \
     './Verif1/S8D' w lp t'S8D - measured'

~~~ 

~~~gnuplot Hydrogaph
reset
set term png
set output 'hydro.png'
load '/Users/geoffroy/mybin/gpl'
set title ''
set xlabel 'T(s)' 
set ylabel 'Inflow (cumsec)' 
set xtics; set ytics
set key bot
plot './hydro.asc' w l t'hydrograph'

*/
/**
## Link to the homepage
* [Homepage](http://basilisk.fr/sandbox/B-flood/Readme)	    		  
   		  
*/